import { SubstrateQueryConfig, ExtrinsicConfig, ContractConfig } from '@moonbeam-network/xcm-builder';
import { IConfigService, AssetConfig, TransferConfig, FeeAssetConfig, DestinationFeeConfig } from '@moonbeam-network/xcm-config';
import { AnyParachain, Asset, AssetAmount, ChainAssetId, AnyChain, EvmParachain, Ecosystem } from '@moonbeam-network/xcm-types';
import { ApiPromise } from '@polkadot/api';
import { Signer } from '@polkadot/api/types';
import { IKeyringPair } from '@polkadot/types/types';
import { Signer as Signer$1 } from 'ethers';
import { WalletClient } from 'viem';

declare class PolkadotService {
    #private;
    readonly api: ApiPromise;
    readonly chain: AnyParachain;
    readonly configService: IConfigService;
    constructor(api: ApiPromise, chain: AnyParachain, configService: IConfigService);
    static create(chain: AnyParachain, configService: IConfigService): Promise<PolkadotService>;
    static createMulti(chains: AnyParachain[], configService: IConfigService): Promise<PolkadotService[]>;
    get decimals(): number;
    get asset(): Asset;
    get existentialDeposit(): AssetAmount;
    getAssetMeta(asset: ChainAssetId): Promise<{
        symbol: string;
        decimals: number;
    } | undefined>;
    getAssetDecimalsFromQuery(asset: ChainAssetId): Promise<number | undefined>;
    getAssetDecimals(asset: Asset): Promise<number>;
    query(config: SubstrateQueryConfig): Promise<bigint>;
    getFee(account: string, config: ExtrinsicConfig): Promise<bigint>;
    transfer(account: string, config: ExtrinsicConfig, signer: Signer | IKeyringPair): Promise<string>;
}

type EvmSigner = Signer$1 | WalletClient;
interface Signers {
    /**
     * @deprecated ethersSigner - is deprecated and will be removed in v2, use evmSigner instead
     */
    ethersSigner?: Signer$1;
    evmSigner?: EvmSigner;
    polkadotSigner: Signer | IKeyringPair;
}
interface TransferData {
    destination: DestinationChainTransferData;
    getEstimate(amount: number | string): AssetAmount;
    isSwapPossible: boolean;
    max: AssetAmount;
    min: AssetAmount;
    source: SourceChainTransferData;
    swap(): Promise<TransferData | undefined>;
    transfer(amount: bigint | number | string): Promise<string>;
}
interface SourceChainTransferData extends ChainTransferData {
    destinationFeeBalance: AssetAmount;
    feeBalance: AssetAmount;
    max: AssetAmount;
}
interface DestinationChainTransferData extends ChainTransferData {
}
interface ChainTransferData {
    balance: AssetAmount;
    chain: AnyChain;
    existentialDeposit: AssetAmount;
    fee: AssetAmount;
    min: AssetAmount;
}

interface GetBalancesParams {
    address: string;
    asset?: Asset;
    chain: AnyChain;
    config: AssetConfig;
    decimals: number;
    polkadot: PolkadotService;
}
type GetDecimalsParams = Omit<GetBalancesParams, 'decimals'> & {
    assetBuiltConfig?: SubstrateQueryConfig | ContractConfig;
};
declare function getBalance({ address, chain, config, decimals, polkadot, }: GetBalancesParams): Promise<bigint>;
declare function getDecimals({ address, asset, config, polkadot, chain, assetBuiltConfig, }: GetDecimalsParams): Promise<number>;
declare function getMin(config: AssetConfig, polkadot: PolkadotService): Promise<bigint>;

interface GetSourceDataParams {
    transferConfig: TransferConfig;
    destinationAddress: string;
    destinationFee: AssetAmount;
    evmSigner?: EvmSigner;
    polkadot: PolkadotService;
    sourceAddress: string;
}
declare function getSourceData({ transferConfig, destinationAddress, destinationFee, polkadot, sourceAddress, evmSigner, }: GetSourceDataParams): Promise<SourceChainTransferData>;
interface GetFeeBalanceParams extends Omit<GetBalancesParams, 'config' | 'evmSigner'> {
    balance: bigint;
    feeConfig: FeeAssetConfig | undefined;
}
declare function getFeeBalance({ address, balance, chain, decimals, feeConfig, polkadot, }: GetFeeBalanceParams): Promise<bigint>;
interface GetFeeParams {
    balance: bigint;
    contract?: ContractConfig;
    chain: AnyChain;
    decimals: number;
    evmSigner?: EvmSigner;
    extrinsic?: ExtrinsicConfig;
    feeConfig?: FeeAssetConfig;
    destinationFeeConfig?: DestinationFeeConfig;
    destinationFeeBalanceAmount?: AssetAmount;
    polkadot: PolkadotService;
    sourceAddress: string;
}
declare function getFee({ balance, chain, contract, decimals, destinationFeeConfig, destinationFeeBalanceAmount, evmSigner, extrinsic, feeConfig, polkadot, sourceAddress, }: GetFeeParams): Promise<bigint>;
declare function getContractFee({ balance, config, decimals, evmSigner, chain, }: {
    balance: bigint;
    config: ContractConfig;
    decimals: number;
    evmSigner: EvmSigner;
    chain: EvmParachain;
}): Promise<bigint>;
declare function getExtrinsicFee(balance: bigint, extrinsic: ExtrinsicConfig, polkadot: PolkadotService, sourceAddress: string): Promise<bigint>;
interface GetMaxParams {
    balanceAmount: AssetAmount;
    existentialDeposit: AssetAmount;
    feeAmount: AssetAmount;
    minAmount: AssetAmount;
}
declare function getMax({ balanceAmount, existentialDeposit, feeAmount, minAmount, }: GetMaxParams): AssetAmount;
interface GetAssetsBalancesParams {
    address: string;
    chain: AnyChain;
    assets: AssetConfig[];
    evmSigner?: EvmSigner;
    polkadot: PolkadotService;
}
declare function getAssetsBalances({ address, chain, assets, polkadot, }: GetAssetsBalancesParams): Promise<AssetAmount[]>;

interface SdkOptions extends Partial<Signers> {
    configService?: IConfigService;
}
declare function Sdk(options?: SdkOptions): {
    assets(ecosystem?: Ecosystem): {
        assets: Asset[];
        asset(keyOrAsset: string | Asset): {
            sourceChains: AnyChain[];
            source(keyOrChain: string | AnyChain): {
                destinationChains: AnyChain[];
                destination(destKeyOrChain: string | AnyChain): {
                    accounts(sourceAddress: string, destinationAddress: string, signers?: Partial<Signers>): Promise<TransferData>;
                };
            };
        };
    };
    getTransferData({ destinationAddress, destinationKeyOrChain, ethersSigner, evmSigner, keyOrAsset, polkadotSigner, sourceAddress, sourceKeyOrChain, }: SdkTransferParams): Promise<TransferData>;
};
declare function getParachainBalances(chain: AnyChain, address: string): Promise<AssetAmount[]>;
interface SdkTransferParams extends Partial<Signers> {
    destinationAddress: string;
    destinationKeyOrChain: string | AnyChain;
    keyOrAsset: string | Asset;
    sourceAddress: string;
    sourceKeyOrChain: string | AnyChain;
}

export { type ChainTransferData, type DestinationChainTransferData, type EvmSigner, type GetAssetsBalancesParams, type GetBalancesParams, type GetDecimalsParams, type GetFeeBalanceParams, type GetFeeParams, type GetMaxParams, type GetSourceDataParams, Sdk, type SdkOptions, type SdkTransferParams, type Signers, type SourceChainTransferData, type TransferData, getAssetsBalances, getBalance, getContractFee, getDecimals, getExtrinsicFee, getFee, getFeeBalance, getMax, getMin, getParachainBalances, getSourceData };
